package com.abi.qms.platform.infrastructure.util;

import com.abi.qms.platform.dao.tablestore.entity.QrCode;
import com.abi.qms.platform.infrastructure.mq.TableStoreBatchInsertConsumer;
import com.abi.qms.platform.infrastructure.mq.TableStoreBatchUpdateConsumer;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.apache.poi.ss.formula.functions.T;
import org.springframework.amqp.core.AmqpTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * 异步写入tablestore工具类
 * @author AndytTan
 */
@Component
public class AsyncTableStoreUtil {

    @Autowired
    private AmqpTemplate amqpTemplate;

    /**
     * 批量新增tableStore,码表
     */
    public <T>  void mqPutRow(List<QrCode> entityList) {
        // 开始多线程写tableStore
        int repeatTimes = entityList.size() / 200;
        int leftover = entityList.size() % 200;
        for (int i = 0; i < repeatTimes+1; i++) {
            if (i == repeatTimes){
                amqpTemplate.convertAndSend(TableStoreBatchInsertConsumer.TABLE_STORE_BATCH_INSERT_QUEUE, entityList.subList(i*200,i*200+leftover));
                continue;
            }
            amqpTemplate.convertAndSend(TableStoreBatchInsertConsumer.TABLE_STORE_BATCH_INSERT_QUEUE, entityList.subList(i*200,i*200+200));
        }
    }

    /**
     * 批量异步更新tablestore,码表
     * @param entityList
     * @param updateColumnFieldName
     * @param <T>
     */
    public <T>  void mqUpdateRow(List<QrCode> entityList,String updateColumnFieldName) {
        // 开始多线程写tableStore
        int repeatTimes = entityList.size() / 200;
        int leftover = entityList.size() % 200;
        for (int i = 0; i < repeatTimes+1; i++) {
            if (i == repeatTimes){
                UpdateRowBean updateRowBean = new UpdateRowBean();
                updateRowBean.setEntityList(entityList.subList(i*200,i*200+leftover));
                updateRowBean.setUpdateColumnFieldName(updateColumnFieldName);

                amqpTemplate.convertAndSend(TableStoreBatchUpdateConsumer.TABLE_STORE_BATCH_UPDATE_QUEUE, updateRowBean);
                continue;
            }

            UpdateRowBean updateRowBean = new UpdateRowBean();
            updateRowBean.setEntityList(entityList.subList(i*200,i*200+200));
            updateRowBean.setUpdateColumnFieldName(updateColumnFieldName);
            amqpTemplate.convertAndSend(TableStoreBatchUpdateConsumer.TABLE_STORE_BATCH_UPDATE_QUEUE, updateRowBean);
        }
    }

    /**
    * 更新行对象
    */
    @Data
    public static class UpdateRowBean implements Serializable {
        private List<QrCode> entityList;
        private String updateColumnFieldName;
    }

}
